@@ -5,13 +5,12 @@
  * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
  * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
  *                                                                  *
- * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2007             *
+ * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2009             *
  * by the Xiph.Org Foundation http://www.xiph.org/                  *
  *                                                                  *
  ********************************************************************
 
- function: 11kHz settings 
- last mod: $Id: setup_11.h 15531 2008-11-24 23:50:06Z xiphmont $
+ function: 11kHz settings
 
  ********************************************************************/
 
@@ -21,8 +20,11 @@ static const int blocksize_11[2]={
   512,512
 };
 
-static const int _floor_mapping_11[2]={
-  6,6,
+static const int _floor_mapping_11a[]={
+  6,6
+};
+static const int *_floor_mapping_11[]={
+  _floor_mapping_11a
 };
 
 static const double rate_mapping_11[3]={
@@ -44,7 +46,7 @@ static const ve_setup_data_template ve_setup_11_stereo={
   2,
   9000,
   15000,
-  
+
   blocksize_11,
   blocksize_11,
 
@@ -62,7 +64,7 @@ static const ve_setup_data_template ve_setup_11_stereo={
   NULL,
   NULL,
   _psy_noise_suppress,
-  
+
   _psy_compand_8,
   _psy_compand_8_mapping,
   NULL,
@@ -73,7 +75,7 @@ static const ve_setup_data_template ve_setup_11_stereo={
 
   _psy_ath_floater_8,
   _psy_ath_abs_8,
-  
+
   _psy_lowpass_11,
 
   _psy_global_44,
@@ -82,8 +84,8 @@ static const ve_setup_data_template ve_setup_11_stereo={
 
   _floor_books,
   _floor,
+  1,
   _floor_mapping_11,
-  NULL,
 
   _mapres_template_8_stereo
 };
@@ -95,7 +97,7 @@ static const ve_setup_data_template ve_setup_11_uncoupled={
   -1,
   9000,
   15000,
-  
+
   blocksize_11,
   blocksize_11,
 
@@ -113,7 +115,7 @@ static const ve_setup_data_template ve_setup_11_uncoupled={
   NULL,
   NULL,
   _psy_noise_suppress,
-  
+
   _psy_compand_8,
   _psy_compand_8_mapping,
   NULL,
@@ -124,7 +126,7 @@ static const ve_setup_data_template ve_setup_11_uncoupled={
 
   _psy_ath_floater_8,
   _psy_ath_abs_8,
-  
+
   _psy_lowpass_11,
 
   _psy_global_44,
@@ -133,9 +135,8 @@ static const ve_setup_data_template ve_setup_11_uncoupled={
 
   _floor_books,
   _floor,
+  1,
   _floor_mapping_11,
-  NULL,
 
   _mapres_template_8_uncoupled
 };
-
